@@ -125,6 +125,8 @@ struct _EmpathyRosterWindowPriv {
   GtkWidget *no_entry_label;
   GtkWidget *button_account_settings;
   GtkWidget *button_online;
+  GtkWidget *button_show_offline;
+  GtkWidget *button_add_contact;
   GtkWidget *spinner_loading;
   GtkWidget *tooltip_widget;
 
@@ -393,14 +395,56 @@ button_online_clicked_cb (GtkButton *button,
   g_object_unref (mgr);
 }
 
+static void
+button_show_offline_clicked_cb (GtkButton *button,
+    EmpathyRosterWindow *self)
+{
+  g_settings_set_boolean (self->priv->gsettings_ui,
+      EMPATHY_PREFS_UI_SHOW_OFFLINE, TRUE);
+}
+
+static void
+button_add_contact_clicked_cb (GtkButton *button,
+    EmpathyRosterWindow *self)
+{
+  empathy_new_individual_dialog_show (GTK_WINDOW (self));
+}
+
 typedef enum
 {
   PAGE_MESSAGE_FLAG_NONE = 0,
   PAGE_MESSAGE_FLAG_ACCOUNTS = 1 << 0,
   PAGE_MESSAGE_FLAG_SPINNER = 1 << 2,
   PAGE_MESSAGE_FLAG_ONLINE = 1 << 3,
+  PAGE_MESSAGE_FLAG_SHOW_OFFLINE = 1 << 4,
+  PAGE_MESSAGE_FLAG_ADD_CONTACT = 1 << 5,
 } PageMessageFlags;
 
+static gboolean
+can_add_contact (EmpathyRosterWindow *self)
+{
+  GList *accounts, *l;
+  gboolean result = FALSE;
+
+  accounts = tp_account_manager_dup_valid_accounts (
+      self->priv->account_manager);
+  for (l = accounts; l != NULL && !result; l = g_list_next (l))
+    {
+      TpAccount *account = TP_ACCOUNT (l->data);
+      TpConnection *conn;
+
+      conn = tp_account_get_connection (account);
+      if (conn == NULL)
+        continue;
+
+      if (tp_connection_get_can_change_contact_list (conn))
+        result = TRUE;
+    }
+
+  g_list_free_full (accounts, g_object_unref);
+  return result;
+}
+
 static void
 display_page_message (EmpathyRosterWindow *self,
     const gchar *msg,
@@ -429,6 +473,14 @@ display_page_message (EmpathyRosterWindow *self,
       (flags & PAGE_MESSAGE_FLAG_SPINNER) != 0);
   gtk_widget_set_visible (self->priv->button_online,
       (flags & PAGE_MESSAGE_FLAG_ONLINE) != 0);
+  gtk_widget_set_visible (self->priv->button_show_offline,
+      (flags & PAGE_MESSAGE_FLAG_SHOW_OFFLINE) != 0);
+  gtk_widget_set_visible (self->priv->button_add_contact,
+      (flags & PAGE_MESSAGE_FLAG_ADD_CONTACT) != 0);
+
+  if ((flags & PAGE_MESSAGE_FLAG_ADD_CONTACT) != 0)
+    gtk_widget_set_sensitive (self->priv->button_add_contact,
+        can_add_contact (self));
 
   gtk_notebook_set_current_page (GTK_NOTEBOOK (self->priv->notebook),
       PAGE_MESSAGE);
@@ -559,6 +611,7 @@ roster_window_error_create_info_bar (EmpathyRosterWindow *self,
   label = gtk_label_new (message_markup);
   gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
   gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+  gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_widget_show (label);
 
@@ -715,6 +768,7 @@ roster_window_update_status (EmpathyRosterWindow *self)
 {
   gboolean connected, connecting;
   GList *l;
+  GAction *action;
 
   connected = empathy_account_manager_get_accounts_connected (&connecting);
 
@@ -733,6 +787,10 @@ roster_window_update_status (EmpathyRosterWindow *self)
   /* Update widgets sensibility */
   for (l = self->priv->actions_connected; l; l = l->next)
     g_simple_action_set_enabled (l->data, connected);
+
+  action = g_action_map_lookup_action (G_ACTION_MAP (self), "chat_add_contact");
+  if (!can_add_contact (self))
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
 }
 
 static void
@@ -990,13 +1048,17 @@ empathy_roster_window_finalize (GObject *window)
 static gboolean
 roster_window_key_press_event_cb  (GtkWidget *window,
     GdkEventKey *event,
-    gpointer user_data)
+    EmpathyRosterWindow *self)
 {
   if (event->keyval == GDK_KEY_T
       && event->state & GDK_SHIFT_MASK
       && event->state & GDK_CONTROL_MASK)
     empathy_chat_manager_call_undo_closed_chat ();
 
+  if (event->keyval == GDK_KEY_f
+      && event->state & GDK_CONTROL_MASK)
+    gtk_widget_show (self->priv->search_bar);
+
   return FALSE;
 }
 
@@ -1625,8 +1687,13 @@ set_notebook_page (EmpathyRosterWindow *self)
         }
       else
         {
-          display_page_message (self, _("No online contacts"),
-              PAGE_MESSAGE_FLAG_NONE);
+          if (g_settings_get_boolean (self->priv->gsettings_ui,
+                EMPATHY_PREFS_UI_SHOW_OFFLINE))
+            display_page_message (self, _("You haven't added any contact yet"),
+                PAGE_MESSAGE_FLAG_ADD_CONTACT);
+          else
+            display_page_message (self, _("No online contacts"),
+                PAGE_MESSAGE_FLAG_SHOW_OFFLINE);
         }
       goto out;
     }
@@ -2144,6 +2211,14 @@ roster_window_most_available_presence_changed_cb (TpAccountManager *manager,
   set_notebook_page (self);
 }
 
+static void
+show_offline_changed_cb (GSettings *settings,
+    const gchar *key,
+    EmpathyRosterWindow *self)
+{
+  set_notebook_page (self);
+}
+
 static void
 empathy_roster_window_init (EmpathyRosterWindow *self)
 {
@@ -2185,6 +2260,8 @@ empathy_roster_window_init (EmpathyRosterWindow *self)
       "roster_scrolledwindow", &sw,
       "button_account_settings", &self->priv->button_account_settings,
       "button_online", &self->priv->button_online,
+      "button_show_offline", &self->priv->button_show_offline,
+      "button_add_contact", &self->priv->button_add_contact,
       "spinner_loading", &self->priv->spinner_loading,
       NULL);
   g_free (filename);
@@ -2193,7 +2270,7 @@ empathy_roster_window_init (EmpathyRosterWindow *self)
   gtk_widget_show (self->priv->main_vbox);
 
   g_signal_connect (self, "key-press-event",
-      G_CALLBACK (roster_window_key_press_event_cb), NULL);
+      G_CALLBACK (roster_window_key_press_event_cb), self);
 
   g_object_unref (gui);
 
@@ -2336,6 +2413,9 @@ empathy_roster_window_init (EmpathyRosterWindow *self)
   g_settings_bind (self->priv->gsettings_ui, EMPATHY_PREFS_UI_SHOW_OFFLINE,
       self->priv->view, "show-offline",
       G_SETTINGS_BIND_GET);
+  tp_g_signal_connect_object (self->priv->gsettings_ui,
+      "changed::" EMPATHY_PREFS_UI_SHOW_OFFLINE,
+      G_CALLBACK (show_offline_changed_cb), self, 0);
   g_settings_bind (self->priv->gsettings_ui, EMPATHY_PREFS_UI_SHOW_GROUPS,
       self->priv->view, "show-groups",
       G_SETTINGS_BIND_GET);
@@ -2347,6 +2427,10 @@ empathy_roster_window_init (EmpathyRosterWindow *self)
       G_CALLBACK (button_account_settings_clicked_cb), self);
   g_signal_connect (self->priv->button_online, "clicked",
       G_CALLBACK (button_online_clicked_cb), self);
+  g_signal_connect (self->priv->button_show_offline, "clicked",
+      G_CALLBACK (button_show_offline_clicked_cb), self);
+  g_signal_connect (self->priv->button_add_contact, "clicked",
+      G_CALLBACK (button_add_contact_clicked_cb), self);
 }
 
 GtkWidget *
